//
//  AppDelegate.swift
//  player
//
//  Created by changsanjiang on 09/16/2022.
//  Copyright (c) 2022 changsanjiang. All rights reserved.
//

import UIKit
import SJVideoPlayer

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplicationLaunchOptionsKey: Any]?) -> Bool {
        let tabBarController = TabBarController.init()
        tabBarController.addChildViewController(NavigationController.init(rootViewController: ViewController()))
        
        window = UIWindow.init(frame: UIScreen.main.bounds)
        window?.backgroundColor = .white
        window?.rootViewController = tabBarController
        window?.makeKeyAndVisible()
        
        // Override point for customization after application launch.
        return true
    }
    
    func application(_ application: UIApplication, supportedInterfaceOrientationsFor window: UIWindow?) -> UIInterfaceOrientationMask {
        print(window)
        if #available(iOS 16.0, *) {
            return window is SJRotationFullscreenWindow ? SJRotationManager.supportedInterfaceOrientations(for: window) : .all
        } else {
            return .all
        }
    }
}

class TabBarController: UITabBarController {
    override var shouldAutorotate: Bool {
        return selectedViewController?.shouldAutorotate ?? false
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return selectedViewController?.supportedInterfaceOrientations ?? .portrait
    }
    
    override var preferredInterfaceOrientationForPresentation: UIInterfaceOrientation {
        return selectedViewController?.preferredInterfaceOrientationForPresentation ?? .portrait
    }
}

class NavigationController: UINavigationController {
    override var shouldAutorotate: Bool {
        return topViewController?.shouldAutorotate ?? false
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return topViewController?.supportedInterfaceOrientations ?? .portrait
    }
    
    override var preferredInterfaceOrientationForPresentation: UIInterfaceOrientation {
        return topViewController?.preferredInterfaceOrientationForPresentation ?? .portrait
    }
    
    override var childViewControllerForStatusBarStyle: UIViewController? {
        return topViewController
    }
    
    override var childViewControllerForStatusBarHidden: UIViewController? {
        return topViewController
    }
}

class ViewController: UIViewController {
    override func viewDidLoad() {
        super.viewDidLoad()
        title = "Demo"
        view.backgroundColor = .white
        
        view.addSubview({
            let button = UIButton(type: .system)
            button.setTitle("Push-isViewControllerRotatable-false", for: .normal)
            button.frame = .init(x: 20, y: 120, width: 300, height: 32)
            button.addBlockAction { [weak self] in
                guard let self = self else { return }
                self.navigationController?.pushViewController(PlayerViewController(isRotatable: false), animated: true)
            }
            return button
        }())
        
        view.addSubview({
            let button = UIButton(type: .system)
            button.setTitle("Push-isViewControllerRotatable-true", for: .normal)
            button.frame = .init(x: 20, y: 200, width: 300, height: 32)
            button.addBlockAction { [weak self] in
                guard let self = self else { return }
                self.navigationController?.pushViewController(PlayerViewController(isRotatable: true), animated: true)
            }
            return button
        }())
    }
    
    override var shouldAutorotate: Bool {
        return false
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return .portrait
    }
    
    override var preferredInterfaceOrientationForPresentation: UIInterfaceOrientation {
        return .portrait
    }
}

class PlayerViewController: UITableViewController {
    /// vc 是否可以旋转
    let isViewControllerRotatable: Bool
    
    init(isRotatable: Bool) {
        isViewControllerRotatable = isRotatable
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        isViewControllerRotatable = false
        super.init(coder: coder)
    }

    private lazy var player: SJVideoPlayer = {
        let vp = SJVideoPlayer()
        vp.onlyFitOnScreen = self.isViewControllerRotatable
        return vp
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        view.backgroundColor = .white
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: "A")
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        player.vc_viewDidAppear()
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        player.vc_viewWillDisappear()
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        player.vc_viewDidDisappear()
    }
    
    override var shouldAutorotate: Bool {
        return isViewControllerRotatable
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return isViewControllerRotatable ? .all : .portrait
    }
    
    override var preferredInterfaceOrientationForPresentation: UIInterfaceOrientation {
        return .portrait
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        20
    }

    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        300
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = UITableViewCell(style: .default, reuseIdentifier: "A")
        cell.textLabel?.text = "点击进行播放"
        cell.textLabel?.textAlignment = .center
        return cell
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        // player.urlAsset = SJVideoPlayerURLAsset(url: .init(string: "https://dh2.v.netease.com/2017/cg/fxtpty.mp4")!)
        
        let url = URL.init(string: "https://dh2.v.netease.com/2017/cg/fxtpty.mp4")!
        let playModel = SJPlayModel.init(tableView: tableView, indexPath: indexPath)
        playModel.superviewSelector = #selector(getter: UITableViewCell.contentView)
        
        player.urlAsset = SJVideoPlayerURLAsset(url: url, playModel: playModel)
    }

    var transitionCount = 0
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)
        guard isViewControllerRotatable == false else { return }
        transitionCount += 1
        self.navigationItem.title = "transitionCount:\(transitionCount)"
        if transitionCount > 20 {
            /* 注释掉
             SJRotationManager_iOS_16_Later.m:95 的 `[sourceWindow makeKeyWindow];`
             和
             SJRotationManager_iOS_16_Later.m:98 的 `[self.window setHidden:YES];`
             就可以避免viewWillTransition,但是功能上就会出问题
             */
            fatalError("点击返回/直接旋转回portrait, 实际上并没有发生Transition，但是会导致UITableView的发生位移")
        }
    }
}

extension UIButton {
    func addBlockAction(block: @escaping () -> Void) {
        let handler = ActionHandler(block: block)
        addTarget(handler, action: #selector(ActionHandler.performAction), for: .touchUpInside)
        objc_setAssociatedObject(self, &UIButton.kActionHandler, handler, .OBJC_ASSOCIATION_RETAIN_NONATOMIC)
    }
    
    private static var kActionHandler: String?
    
    private class ActionHandler {
        let block: () -> Void
        init(block: @escaping () -> Void) {
            self.block = block
        }
        
        @objc func performAction() {
            block()
        }
    }
}
